/* main.c */

/**************************************************************************+
*
*  Code for generating colliding messages for PANAMA, as specified in 
*  the paper 'Generating Collisions for Panama, instantaneously' by Joan 
*  Daemen and Gilles Van Assche, presented at the Fast Software Encryption 
*  Workshop, Luxembourg, 2007, see "Proceedings of FSE 2007", edited by 
*  Alex Biryukov, Springer-Verlag, to appear.
*  This code is made available on http://radiogatun.noekeon.org/panama
*
*  Code by Joan Daemen
*
*  Notes:  This code is supplied for the purposes of illustrating 
*          the collision generating attack only 
*
+**************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "panama.h"

#define Obar 0xffffffff

int main(int argc, char *argv[])
{
int i, j, offset, dummy;
UINT32 *D;
UINT32 *M;
UINT32 hashcode1[8*32], hashcode2[8*32];

if (sizeof(UINT32) != 4) {
    printf("UINT32 is not 32-bit wide!");
    abort();
}

/* M: random message, mypanama_collision adapts it to contain member of colliding pair */

printf("give number of dummy calls to rand(), 0 gives all-zero message\n");
scanf("%d",&dummy);
printf("give offset (message block index where first difference appears)\n");
scanf("%d",&offset);

M = (UINT32 *) malloc(8*(offset+36)*sizeof(UINT32));
D = (UINT32 *) malloc(8*(offset+36)*sizeof(UINT32));

for (i=0 ; i<dummy ; i++) rand(); 
for (i=0 ; i<8*(offset+36) ; i++) M[i] = 0;
if (dummy > 0) {
   for (i=0 ; i<8*(offset+36) ; i++) M[i] = rand() ^ rand()<<16;}

mypanama_collision(M,offset); 
mypanama_generate_difference(D,offset); 
mypanama_collision_printdiff(D,M,offset);

printf("message 1:\n");
for (j=0 ; j<offset+36 ; j++)
    {
    printf("p[%2d]:",j);
    for (i=0 ; i<8 ; i++) printf("%08x ",M[i+(j*8)]);
    printf("\n");
    }
pan_hash(M,hashcode1,32*8*(offset+36));

for (i=0; i<8*(offset+36); i++) M[i] ^= D[i];

printf("message 2:\n");
for (j=0 ; j<offset+36 ; j++)
    {
    printf("p[%2d]:",j);
    for (i=0 ; i<8 ; i++) printf("%08x ",M[i+(j*8)]);
    printf("\n");
    }

pan_hash(M,hashcode2,32*8*(offset+36));

printf("hash of message 1:\n");
for (i=0; i<8; i++) printf("%08x ",hashcode1[i]);
printf("\n");

printf("hash of message 2:\n");
for (i=0; i<8; i++) printf("%08x ",hashcode2[i]);
printf("\n");

}
